@@ -1,4 +1,6 @@
 import logging
+import shutil
+import tempfile
 import os
 from pkg_resources import resource_filename
 
@@ -6,6 +8,8 @@ from cmdstanpy import CmdStanModel
 
 import numpy as np
 import pandas as pd
+import statsmodels.api as sm
+from scipy.stats import norm
 
 # cmdstanpy natrually generates a lot of logs this is to make those logs quiet unless there is an error
 # https://discourse.mc-stan.org/t/cmdstanpy-not-compiling-with-uninformative-error/25576/7?u=mitzimorris
@@ -67,19 +71,22 @@ def fit_stan_model(model_name_string, y, lag: int, uncertainty_samples: int, hor
     """
     Fit the stan model
     """
-    stan_model_file = resource_filename("tablespoon", "stan/" + model_name_string + ".stan")
-    out_dir = resource_filename("tablespoon", "stan/out")
-    if not os.path.exists(out_dir):
-        out_dir = os.path.expanduser(out_dir)
-        os.makedirs(out_dir, exist_ok=True)
-    model_stan = CmdStanModel(stan_file=stan_model_file)
-    cmdstanpy_data = {"horizon": horizon, "T": len(y), "y": y, "lag": lag}
-    fit = model_stan.sample(data=cmdstanpy_data, output_dir=out_dir, chains=1, seed=42, chain_ids=chain_ids, iter_sampling=uncertainty_samples)
-    df_fit = fit.draws_pd()
-    df_fit = df_fit.loc[:, df_fit.columns.str.startswith("forecast")]
-    if verbose:
-        print(fit.summary())
-    return df_fit
+    with tempfile.TemporaryDirectory() as d:
+        temp_dir_file = os.path.join(d, model_name_string + ".stan")
+        stan_model_file = resource_filename("tablespoon", "stan/" + model_name_string + ".stan")
+        shutil.copyfile(stan_model_file, temp_dir_file)
+        out_dir = os.path.join(d, "stan", "out")
+        if not os.path.exists(out_dir):
+            out_dir = os.path.expanduser(out_dir)
+            os.makedirs(out_dir, exist_ok=True)
+        model_stan = CmdStanModel(stan_file=temp_dir_file)
+        cmdstanpy_data = {"horizon": horizon, "T": len(y), "y": y, "lag": lag}
+        fit = model_stan.sample(data=cmdstanpy_data, output_dir=out_dir, chains=1, seed=42, chain_ids=chain_ids, iter_sampling=uncertainty_samples)
+        df_fit = fit.draws_pd()
+        df_fit = df_fit.loc[:, df_fit.columns.str.startswith("forecast")]
+        if verbose:
+            print(fit.summary())
+        return df_fit
 
 
 class Naive(object):
@@ -107,6 +114,7 @@ class Naive(object):
         uncertainty_samples=5000,
         include_history=False,
         chain_ids=None,
+        use_stan_backend=True, 
         verbose=False,
     ):
         """Predict - forecast method
@@ -119,6 +127,7 @@ class Naive(object):
             uncertainty_samples (int, optional): number of uncertainty samples to draw. Defaults to 5000.
             include_history (bool, optional): include history. Defaults to False.
             chain_ids (str, optional): identifiers for chain ids. Defaults to None.
+            use_stan_backend (bool, optional): chose to use either standard python (False) or stan backend (True)
             verbose (bool, optional): verbose. Defaults to False.
 
         Returns:
@@ -139,10 +148,29 @@ class Naive(object):
         df_dates = pd.DataFrame({"ds": dates})
         df_samples = pd.DataFrame({"rep": np.arange(uncertainty_samples)})
         df_cross = df_dates.merge(df_samples, how="cross")
-        df_fit = fit_stan_model("naive", self.y, lag, uncertainty_samples, horizon, chain_ids, verbose=verbose)
-        np_predictions = df_fit.to_numpy().transpose().reshape(uncertainty_samples * horizon, 1)
-        df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
-        df_result = pd.concat([df_cross, df_pred], axis=1)
+        if use_stan_backend:
+            df_fit = fit_stan_model("naive", self.y, lag, uncertainty_samples, horizon, chain_ids, verbose=verbose)
+            #forecast[1] forecast[2]... forecast[28]
+            np_predictions = df_fit.to_numpy().transpose().reshape(uncertainty_samples * horizon, 1)
+            df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
+            df_result = pd.concat([df_cross, df_pred], axis=1)
+        else:
+            t = lag+1
+            t_lag = t-lag
+            y = self.y
+            end = len(y)-lag
+            yt = y[t:].to_numpy()
+            yt_lag = y[t_lag:end].to_numpy()
+            y_last = self.y[len(y)-1]
+            mod = sm.GLM(yt, yt_lag, family=sm.families.Gaussian())
+            sigma = np.sqrt(mod.fit().scale)
+            rng = np.random.default_rng()
+            forecast = np.empty([uncertainty_samples, horizon])
+            for h in range(0, horizon):
+                forecast[:,h] = y_last + sigma * np.sqrt(h+1) * rng.standard_normal(uncertainty_samples)
+            np_predictions = forecast.transpose().reshape(uncertainty_samples * horizon, 1)
+            df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
+            df_result = pd.concat([df_cross, df_pred], axis=1)
         return df_result
 
 
@@ -160,7 +188,7 @@ class Mean(object):
     ):
         self.include_history = include_history
 
-    def predict(self, df_historical, horizon=30, frequency=None, lag=1, uncertainty_samples=5000, include_history=False, chain_ids=None, verbose=False):
+    def predict(self, df_historical, horizon=30, frequency=None, lag=1, uncertainty_samples=5000, include_history=False, chain_ids=None,use_stan_backend=True, verbose=False):
         """Predict - forecast method
 
         Args:
@@ -171,6 +199,7 @@ class Mean(object):
             uncertainty_samples (int, optional): number of uncertainty samples to draw. Defaults to 5000.
             include_history (bool, optional): include history. Defaults to False.
             chain_ids (str, optional): identifiers for chain ids. Defaults to None.
+            use_stan_backend (bool, optional): chose to use either standard python (False) or stan backend (True)
             verbose (bool, optional): verbose. Defaults to False.
 
         Returns:
@@ -189,10 +218,23 @@ class Mean(object):
         df_dates = pd.DataFrame({"ds": dates})
         df_samples = pd.DataFrame({"rep": np.arange(uncertainty_samples)})
         df_cross = df_dates.merge(df_samples, how="cross")
-        df_fit = fit_stan_model("mean", self.y, lag, uncertainty_samples, horizon, chain_ids, verbose=verbose)
-        np_predictions = df_fit.to_numpy().transpose().reshape(uncertainty_samples * horizon, 1)
-        df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
-        df_result = pd.concat([df_cross, df_pred], axis=1)
+        if use_stan_backend:
+            df_fit = fit_stan_model("mean", self.y, lag, uncertainty_samples, horizon, chain_ids, verbose=verbose)
+            np_predictions = df_fit.to_numpy().transpose().reshape(uncertainty_samples * horizon, 1)
+            df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
+            df_result = pd.concat([df_cross, df_pred], axis=1)
+        else:
+            y = self.y
+            T = len(y)
+            deg_freedom = T - 1
+            mu, sigma = norm.fit(y)
+            rng = np.random.default_rng()
+            forecast = np.empty([uncertainty_samples, horizon])
+            for h in range(0, horizon):
+                forecast[:,h] = mu + sigma * np.sqrt(1 + (1 / T)) * rng.standard_t(df = deg_freedom, size = uncertainty_samples)
+            np_predictions = forecast.transpose().reshape(uncertainty_samples * horizon, 1)
+            df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
+            df_result = pd.concat([df_cross, df_pred], axis=1)
         return df_result
 
 
@@ -219,6 +261,7 @@ class Snaive(object):
         uncertainty_samples=5000,
         include_history=False,
         chain_ids=None,
+        use_stan_backend=True, 
         verbose=False,
     ):
         """Predict - forecast method
@@ -231,6 +274,7 @@ class Snaive(object):
             uncertainty_samples (int, optional): number of uncertainty samples to draw. Defaults to 5000.
             include_history (bool, optional): include history. Defaults to False.
             chain_ids (str, optional): identifiers for chain ids. Defaults to None.
+            use_stan_backend (bool, optional): chose to use either standard python (False) or stan backend (True)
             verbose (bool, optional): verbose. Defaults to False.
 
         Returns:
@@ -249,8 +293,26 @@ class Snaive(object):
         df_dates = pd.DataFrame({"ds": dates})
         df_samples = pd.DataFrame({"rep": np.arange(uncertainty_samples)})
         df_cross = df_dates.merge(df_samples, how="cross")
-        df_fit = fit_stan_model("snaive", self.y, lag, uncertainty_samples, horizon, chain_ids, verbose=verbose)
-        np_predictions = df_fit.to_numpy().transpose().reshape(uncertainty_samples * horizon, 1)
-        df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
-        df_result = pd.concat([df_cross, df_pred], axis=1)
-        return df_result
+        if use_stan_backend:
+            df_fit = fit_stan_model("snaive", self.y, lag, uncertainty_samples, horizon, chain_ids, verbose=verbose)
+            np_predictions = df_fit.to_numpy().transpose().reshape(uncertainty_samples * horizon, 1)
+            df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
+            df_result = pd.concat([df_cross, df_pred], axis=1)
+        else:
+            t = lag+1
+            t_lag = t-lag
+            y = self.y
+            end = len(y)-lag
+            yt = y[t:].to_numpy()
+            yt_lag = y[t_lag:end].to_numpy()
+            y_last = self.y[(len(y)-1)-(lag-(horizon%lag))]
+            mod = sm.GLM(yt, yt_lag, family=sm.families.Gaussian())
+            sigma = np.sqrt(mod.fit().scale)
+            rng = np.random.default_rng()
+            forecast = np.empty([uncertainty_samples, horizon])
+            for h in range(0, horizon):
+                forecast[:,h] = y_last + sigma * np.sqrt(np.trunc(((h-1)*1)/(lag)) + 1) * rng.standard_normal(uncertainty_samples)
+            np_predictions = forecast.transpose().reshape(uncertainty_samples * horizon, 1)
+            df_pred = pd.DataFrame(np_predictions, columns=["y_sim"])
+            df_result = pd.concat([df_cross, df_pred], axis=1)
+        return df_result
\ No newline at end of file
